<?php
/*************************************************************** 
*  Copyright notice 
* 
*  (c) 2006 Daniel Schledermann <daniel@typoconsult.dk> 
*  All rights reserved 
* 
*  This script is part of the TYPO3 project. The TYPO3 project is 
*  free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or 
*  (at your option) any later version. 
* 
*  The GNU General Public License can be found at 
*  http://www.gnu.org/copyleft/gpl.html. 
* 
*  This script is distributed in the hope that it will be useful, 
*  but WITHOUT ANY WARRANTY; without even the implied warranty of 
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
*  GNU General Public License for more details. 
* 
*  This copyright notice MUST APPEAR in all copies of the script! 
***************************************************************/


/**
 * This is the mailer core of tcdirectmail. 
 * It is normally used in a single instance to compile MIME 1.0 compatible mails
 */
class tx_tcdirectmail_mailer {
    /* Vars that might need to be overridden */
    var $senderName = "Test";
    var $senderEmail = "test@test.net";
    var $replytoName;
    var $replytoEmail;
    var $domain;
    var $sendid = '';
    var $cryptid = '';

    /**
     * Constructor that set up basic internal datastructures. Do not call directly
     *
     */
    function tx_tcdirectmail_mailer () {
	if( ini_get('safe_mode') ){
    	    $this->hostname = $_SERVER['HTTP_HOST'];
        } else {
	    $this->hostname = trim(`hostname -f`);
	}
    
	$this->realPath = PATH_site;
	$this->inlinefiles = array();
	$this->inlinemimes = array();
	$this->files = array();
	$this->mimes = array();
	$this->boundarys = array('We-come-in-peace','Take-me-to-your-leader','Dont-run-we-are-your-friends',
		'Its-life-Jim--But-not-as-we-know-it','Im-sorry-Dave-Im-afraid-I-cant-do-that','Dont-be-alarmed-at-our-appearance', 
		'Use-the-force-Luke', 'A-xenomorph-may-be-involved', 'Your-culture-will-adapt-to-service-us', 'Resistance-is-futile',
		'The-Force-will-be-with-you--always','Wars-not-make-one-great', 'I-felt-a-great-disturbance-in-the-Force',
		'Im-not-looking-for-a-friend---Im-looking-for-a-Jedi-master',
		'The-best-thing-about-me-is-that-there-is-so-many-of-me', 'You?-Yes-me----me,-me,-me----Me-too');
    }
    
    /**
     * Encodes a text i qouted printable format. Either via build in function, with fallback to a php-implementation.
     *
     * @static
     * @param	string		Text to be encoded
     * @return	string		Encoded text
     */
    
    function qoutedPrintableEncode($ascii_in = "") {
	/* If the imap-extension is installed, use the encoding from there */
	if (function_exists('imap_8bit')) {
	    return imap_8bit($ascii_in);
	    
	/* Otherwise this slower php implementation */
	} else {
	    $ascii_array = explode ("\n", $ascii_in);
	    $dont_replace = array(',',' ');
	    $replaced = array('=2C','=20');
	
	    foreach ($ascii_array as $str) {
		$str = str_replace($replaced, $dont_replace, str_replace("%","=",rawurlencode($str)));
		if (strlen($str) > 75) {
		    $str = wordwrap($str, 75, " =\r\n");
		}
		$ascii_out[] = $str;
	    }
	
	    return implode ("\n", $ascii_out);
	}
    }

    /**
     * Determine the charset encoding of the mail.
     *
     * @internal
     * @return	string		Determined charset encoding
     */
    function getCharsetEncoding() {
	/* Does the code come with anything? */
	if (preg_match ('|<meta http-equiv="Content-Type" content="text/html; charset=([^"]*)" />|', $this->html_tpl, $match)) {
	    return $match[1];
	}
	
	/* Is anything provided in TYPO3_CONF_VARS? */
	if (trim($GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset'])) {
	    return $GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset'];
	}
	
	/* Oh well..  We'll just have to guess something reasonable */
	return 'iso-8859-1';
    }

    /**
     * Get mime-type for a file
     *
     * @static
     * @return 	string		Mime type determined.
     */
    function getMimeType($filename) {
	preg_match ('/[^\.]+$/', $filename, $match);
	
	switch ($match[0]) {
	case "zip" 	: return "application/zip";
	case "php"	:
	case "php4"	:
	case "php5"	:
	case "phtml"	:
	case "inc"	: return "application/x-php";
	case "pl"	:
	case "cgi"	: return "application/x-perl";
	case "sql"	:
	case "asc"	:
	case "txt"	: return "text/plain";
	case "dcr"	:
	case "dir"	:
	case "dxr"	: return "application/x-director";
	case "skp"	:
	case "skd"	:
	case "skt"	:
	case "skm"	: return "application/x-koan";
	case "t"  	:
	case "tr" 	: 
	case "roff"	: return "application/x-troff";
	case "mid"	:
	case "midi"	:
	case "kar"	: return "audio/midi";
	case "mpga"	:
	case "mp2" 	:
	case "mp3" 	: return "audio/mpeg";
	case "ai" 	:
	case "eps" 	:
	case "ps"	: return "application/postscript";
	case "smi"	: 
	case "smil"	: return "application/smil";
	case "aif" 	:
	case "aiff" 	:
	case "aifc"	: return "audio/x-aiff";
	case "jpeg" 	:
	case "jpg"  	:
	case "jpe"	: return "image/jpeg";
	case "au" 	:
	case "snd"	: return "audio/basic";
	case "msh"	:
	case "mesh"	:
	case "silo"	: return "model/mesh";
	case "ram" 	:
	case "rm"	: return "audio/x-pn-realaudio";
	case "pdf" 	: return "application/pdf";
	case "xhtml"	:
	case "xht"	: return "application/xhtml+xml";
	case "html" 	:
	case "tmpl"	:
	case "htm"	: return "text/html";
	case "tiff" 	:
	case "tif"  	: return "image/tiff";
	case "xml"	:
	case "xsl"	: return "text/xml";
	case "djvu" 	:	
	case "djv"	: return "image/vnd.djvu";
	case "wrl" 	:
	case "vrml" 	: return "model/vrml";
	case "mpeg" 	:
	case "mpg"	:
	case "mpe"	: return "video/mpeg";
	case "qt" 	:
	case "mov" 	: return "video/quicktime";
	case "ez" 	: return "application/andrew-inset";
	case "hqx" 	: return "application/mac-binhex40";
	case "cpt" 	: return  "application/mac-compactpro";
	case "oda" 	: return  "application/oda";
	case "wbxml" 	: return  "application/vnd.wap.wbxml";
	case "wmlc" 	: return  "application/vnd.wap.wmlc";
	case "wmlsc" 	: return  "application/vnd.wap.wmlscriptc";
	case "bcpio" 	: return  "application/x-bcpio";
	case "vcd" 	: return  "application/x-cdlink";
	case "pgn" 	: return  "application/x-chess-pgn";
	case "cpio" 	: return  "application/x-cpio";
	case "csh" 	: return  "application/x-csh";
	case "dvi" 	: return  "application/x-dvi";
	case "spl" 	: return  "application/x-futuresplash";
	case "gtar" 	: return  "application/x-gtar";
	case "hdf" 	: return  "application/x-hdf";
	case "js" 	: return  "application/x-javascript";
	case "latex" 	: return  "application/x-latex";
	case "nc" 	: return  "application/x-netcdf";
	case "cdf" 	: return  "application/x-netcdf";
	case "sh" 	: return  "application/x-sh";
	case "shar" 	: return  "application/x-shar";
	case "swf" 	: return  "application/x-shockwave-flash";
	case "sit" 	: return  "application/x-stuffit";
	case "sv4cpio" 	: return  "application/x-sv4cpio";
	case "sv4crc" 	: return  "application/x-sv4crc";
	case "tar" 	: return  "application/x-tar";
	case "tcl" 	: return  "application/x-tcl";
	case "tex" 	: return  "application/x-tex";
	case "texinfo" 	: return  "application/x-texinfo";
	case "texi" 	: return  "application/x-texinfo";
	case "man" 	: return  "application/x-troff-man";
	case "me" 	: return  "application/x-troff-me";
	case "ms" 	: return  "application/x-troff-ms";
	case "ustar" 	: return  "application/x-ustar";
	case "src" 	: return  "application/x-wais-source";
	case "bz2" 	: return  "application/x-bzip2";
	case "gz" 	: return  "application/x-gzip";
	case "m3u" 	: return  "audio/x-mpegurl";
	case "rpm" 	: return  "application/x-rpm";
	case "ra" 	: return  "audio/x-realaudio";
	case "wav" 	: return  "audio/x-wav";
	case "pdb" 	: return  "chemical/x-pdb";
	case "xyz" 	: return  "chemical/x-xyz";
	case "bmp" 	: return  "image/bmp";
	case "gif" 	: return  "image/gif";
	case "ief" 	: return  "image/ief";
	case "png" 	: return  "image/png";
	case "wbmp"	: return  "image/vnd.wap.wbmp";
	case "ras" 	: return  "image/x-cmu-raster";
	case "pnm" 	: return  "image/x-portable-anymap";
	case "pbm" 	: return  "image/x-portable-bitmap";
	case "pgm" 	: return  "image/x-portable-graymap";
	case "ppm" 	: return  "image/x-portable-pixmap";
	case "rgb" 	: return  "image/x-rgb";
	case "xbm" 	: return  "image/x-xbitmap";
	case "xpm" 	: return  "image/x-xpixmap";
	case "xwd" 	: return  "image/x-windowdump";
	case "igs" 	:
	case "iges"	: return  "model/iges";
	case "css" 	: return  "text/css";
	case "rtx" 	: return  "text/richtext";
	case "rtf" 	: return  "text/rtf";
	case "sgml" 	: return  "text/sgml";
	case "sgm" 	: return  "text/sgml";
	case "tsv" 	: return  "text/tab-seperated-values";
	case "wml" 	: return  "text/vnd.wap.wml";
	case "wmls" 	: return  "text/vnd.wap.wmlscript";
	case "etx" 	: return  "text/x-setext";
	case "mxu" 	: return  "video/vnd.mpegurl";
	case "avi" 	: return  "video/x-msvideo";
	case "movie" 	: return  "video/x-sgi-movie";
	case "ice" 	: return  "x-conference-xcooltalk";
	case "asf"	:
	case "asx"	: return  "video/x-ms-asf";
	case "wma"	: return  "audio/x-ms-wma";
	case "wax"	: return  "audio/x-ms-wax";
	case "wmv"	: return  "audio/x-ms-wmv";
	case "wvx"	: return  "video/x-ms-wvx";
	case "wm"	: return  "video/x-ms-wm";
	case "wmx"	: return  "video/x-ms-wmx";
	case "wmz"	: return  "application/x-ms-wmz";
	case "wmd"	: return  "application/x-ms-wmd";
  	case "odt"	: return  "application/vnd.oasis.opendocument.text";
	case "ott"	: return  "application/vnd.oasis.opendocument.text-template";
  	case "oth"	: return  "application/vnd.oasis.opendocument.text-web";
	case "odm"  	: return  "application/vnd.oasis.opendocument.text-master";
	case "odg"  	: return  "application/vnd.oasis.opendocument.graphics";
	case "otg"  	: return  "application/vnd.oasis.opendocument.graphics-template";
  	case "odp" 	: return  "application/vnd.oasis.opendocument.presentation";
	case "otp"  	: return  "application/vnd.oasis.opendocument.presentation-template";
	case "ods"	: return  "application/vnd.oasis.opendocument.spreadsheet";
	case "ots"	: return  "application/vnd.oasis.opendocument.spreadsheet-template";
	case "odc"  	: return  "application/vnd.oasis.opendocument.chart";
	case "odf"  	: return  "application/vnd.oasis.opendocument.formula";
	case "odb"	: return  "application/vnd.oasis.opendocument.database";
	case "odi"  	: return  "application/vnd.oasis.opendocument.image";
	case "sxw"	: return  "application/vnd.sun.xml.writer";
	case "stw"	: return  "application/vnd.sun.xml.writer.template";
	case "sxc"	: return  "application/vnd.sun.xml.calc";
	case "stc"	: return  "application/vnd.sun.xml.calc.template";
	case "sxd"	: return  "application/vnd.sun.xml.draw";
	case "std"	: return  "application/vnd.sun.xml.draw.template";
	case "sxi"	: return  "application/vnd.sun.xml.impress";
	case "sti"	: return  "application/vnd.sun.xml.impress.template";
	case "sxg"	: return  "application/vnd.sun.xml.writer.global";
	case "sxm"	: return  "application/vnd.sun.xml.math";
	case "doc"	: return  "application/msword";
	case "xls"	: return  "application/x-msexcel";
	case "ppt"	: return  "application/vnd.ms-powerpoint";
	case "prz" 	: return  "application/vnd.lotus-freelance";
	case "123"	: return  "application/vnd.lotus-1-2-3";
	case "lwp"	: return  "application/vnd.lotus-wordpro";
	default 	: return  "application/octet-stream";
	}
    }
    
    
    /**
     * Add a file attachement to the mail
     *
     * @param	string		Filename of file to attach.
     * @return	void
     */
    
    function addAttachment($filename) {
	if (trim($filename) != '') {
	    $path = explode ('/', $filename);
	    $basename = array_pop ($path);

	    $this->files[$basename] = base64_encode(file_get_contents($filename));
	    $this->mimes[$basename] = $this->getMimeType($filename);
	}
    }
    

    /**
     * Set the title text of the mail
     *
     * @param	string		The title
     * @return	void
     */
    function setTitle($src) {
	/* Detect what markers we need to substitute later on */
	preg_match_all ('/###[\w]+###/', $src, $fields);
	$this->titleMarkers = str_replace ('###', '', $fields[0]);
	
	/* Any advanced markers we need to sustitute later on */
	$this->titleAdvancedMarkers = array();
	preg_match_all ('/###:IF: (\w+) ###/U', $src, $fields);
	foreach ($fields[1] as $field) {
	    $this->titleAdvancedMarkers[] = $field;
	}
	
	$this->title_tpl = $src;
	$this->title = $src;
    }
    
    
    /**
     * Set the plain text content on the mail
     *
     * @param	string		The plain text content of the mail
     * @return	void
     */
    function setPlain ($src) {
	/* Remove html-comments */
	$src = preg_replace('/<!--.*-->/U', '', $src);
	
	/* Detect what markers we need to substitute later on */
	preg_match_all ('/###[\w]+###/', $src, $fields);
	$this->plainMarkers = str_replace ('###', '', $fields[0]);
	
	/* Any advanced markers we need to sustitute later on */
	$this->plainAdvancedMarkers = array();
	preg_match_all ('/###:IF: (\w+) ###/U', $src, $fields);
	foreach ($fields[1] as $field) {
	    $this->plainAdvancedMarkers[] = $field;
	}
	
	$this->plain_tpl = $src;
	$this->plain = $src;
    }
    
    /**
     * Set the html content on the mail
     *
     * @param	string		The html content of the mail
     * @return	void
     */    
    function setHtml ($src) {
	/* Remove html-comments */
	$src = preg_replace('/<!--.*-->/U', '', $src);
    
	/* Find linked css and convert into a style-tag */
	preg_match_all('|<link rel="stylesheet" type="text/css" href="(.*)" />|Ui', $src, $urls);
	foreach ($urls[1] as $i => $url) {
	    $get_url = str_replace("http://$this->domain/", '', $url);
	    $src = str_replace ($urls[0][$i], 
			"<style type=\"text/css\">\n<!--\n"
			.file_get_contents($this->realPath.str_replace("http://$this->domain/", '', $url))
			."\n-->\n</style>", $src);
	}
	
	/* We cant very well have attached javascript in a newsmail ... removing */
	$src = preg_replace('|<script type="text/javascript" src="[^"]+"></script>|', '', $src);

	/* Convert external file resouces to attached filer or correct their links */
	$replace_regs = array(
	    '/ src="([^"]+)"/',
	    '/ background="([^"]+)"/',
	);
	
	/* Attach */
	if (!$GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['tcdirectmail']['dontEncodeImages']) {
	    foreach ($replace_regs as $replace_reg) {
		preg_match_all($replace_reg, $src, $urls);
		foreach ($urls[1] as $i => $url) {
		    $get_url = str_replace("http://$this->domain/", '', $url);
		    $name = explode ('.', $get_url);
		    $ext = array_pop ($name);
		    $substname = substr(md5($url), 0, 10).'.'.$ext;
		    $this->inlinefiles[$substname] = base64_encode(file_get_contents("$this->realPath$get_url"));
		    $this->inlinemimes[$substname] = $this->getMimeType("$this->realPath$get_url");
		    $src = str_replace ($urls[0][$i], str_replace($url, "cid:$substname", $urls[0][$i]), $src);
		}
	    }
	    
	/* Or correct link */
	} else {
	    foreach ($replace_regs as $replace_reg) {
		preg_match_all($replace_reg, $src, $urls);
    	    	foreach ($urls[1] as $i => $url) {
		    if (!preg_match('|^http://|', $url)) {
			$src = str_replace ($urls[0][$i], 
					    str_replace($url, "http://$this->domain/$url", $urls[0][$i]), 
					    $src);
		    }
		}
	    }
	}	
	
	/* Fix relative links */
	preg_match_all ('|<a [^>]*href="(.*)"|Ui', $src, $urls);
	foreach ($urls[1] as $i => $url) {
		/* If this is already a absolute link, dont replace it */
		if (!preg_match('|^http://|', $url) && !preg_match('|^mailto:|', $url) && !preg_match('|^#|', $url)) {
		    $replace_url = str_replace($url, "http://$this->domain/$url", $urls[0][$i]);
		    $src = str_replace ($urls[0][$i], $replace_url, $src);
		}
	}
	
	/* Detect what markers we need to substitute later on */
	preg_match_all ('/###[\w]+###/', $src, $fields);
	$this->htmlMarkers = str_replace ('###', '', $fields[0]);
	
	/* Any advanced fields we need to sustitute later on */
	$this->htmlAdvancedMarkers = array();
	preg_match_all ('/###:IF: (\w+) ###/U', $src, $fields);
	foreach ($fields[1] as $field) {
	    $this->htmlAdvancedMarkers[] = $field;
	}


    
	$this->html_tpl = $src;
	$this->html = $src;
    }
    
    /**
     * Insert a "mail-open-spy" in the mail for test.
     *
     * @return	void
     */
    function testSpy () {
	$this->html = str_replace (
	                '</body>', 
			'<div><img src="http://'.$this->domain.'/clear.gif" width="0" height="0" /></div></body>', 
			$this->html);
    }
    
    
    /**
     * Insert a "mail-open-spy" in the mail for real. This relies on the $this->cryptid being set.
     *
     * @return	void
     */
    function insertSpy() {
	$this->html = str_replace (
	                '</body>', 
			'<div><img src="http://'.$this->domain.'/typo3conf/ext/tcdirectmail/beenthere.php?cryptid='.$this->cryptid.'" width="0" height="0" /></div></body>', 
			$this->html);
    }
    
    /**
     * Reset all modifications to the content.
     *
     * @return	void
     */
    function resetMarkers() {
	$this->html  = $this->html_tpl;
	$this->plain = $this->plain_tpl;
	$this->title = $this->title_tpl;
    }
    
    /**
     * Replace a named marker with a suppied value. 
     * A marker can have the form of a simple string marker ###marker###
     * Or a advanced boolean marker ###:IF: marker ### ..content.. (###:ELSE:###)? ..content.. ###:ENDIF:###
     *
     * @param	string		Name of the marker to replace
     * @param	string		Value to replace marker with.
     * @return	void
     */
    function substituteMarker($name, $value) {
	/* For each marker, only substitute if the field is registered as a marker. */
	if (in_array($name, $this->htmlAdvancedMarkers)) {
	    $this->html = tx_tcdirectmail_mailer::advancedSubstituteMarker($this->html, $name, $value);
	}

	if (in_array($name, $this->plainAdvancedMarkers)) {
	    $this->plain = tx_tcdirectmail_mailer::advancedSubstituteMarker($this->plain, $name, $value);
	}

	if (in_array($name, $this->titleAdvancedMarkers)) {
	    $this->title = tx_tcdirectmail_mailer::advancedSubstituteMarker($this->title, $name, $value);
	}

	if (in_array($name, $this->htmlMarkers)) {
	    $this->html  = str_replace("###$name###", $value, $this->html);
	}
	
	if (in_array($name, $this->plainMarkers)) {
	    $this->plain = str_replace("###$name###", $value, $this->plain);
	}
	
	if (in_array($name, $this->titleMarkers)) {
	    $this->title = str_replace("###$name###", $value, $this->title);
	}
    }
    
    /**
     * Substitute an advanced marker.
     *
     * @internal
     * @param	string		Source to apply marker substitution to.
     * @param	string		Name of marker.
     * @param	boolean		Display value of marker.
     * @return	string		Source with applied marker.
     */
    function advancedSubstituteMarker ($src, $name, $value) {
	preg_match_all("/###:IF: $name ###([\w\W]*)###:ELSE:###([\w\W]*)###:ENDIF:###/U", $src, $matches);
	foreach ($matches[0] as $i => $full_mark) {
	    if ($value) {
	    	    $src = str_replace($full_mark, $matches[1][$i], $src);
	    } else {
		    $src = str_replace($full_mark, $matches[2][$i], $src);
	    }
	}
	
	preg_match_all("/###:IF: $name ###([\w\W]*)###:ENDIF:###/U", $src, $matches);
	foreach ($matches[0] as $i => $full_mark) {
	    if ($value) {
	        $src = str_replace($full_mark, $matches[1][$i], $src);
	    } else {
	        $src = str_replace($full_mark, '', $src);
	    }
	}
	
	return $src;
    }

    /**
     * Apply multiple markers to mail contents
     *
     * @param	array		Assoc array with name => value pairs.
     * @return	void
     */
    function substituteMarkers ($record) {
	foreach ($record as $name => $value) {
	    $this->substituteMarker($name, $value);
	}
    }
    
    /**
     * Replace all links in the mail to make spy links.
     * This operation requires $this->cryptid to be set.
     *
     * @return	array		Data structure with original links.
     */
    function makeClickLinks () {
	$links['plain'] = array();
	$links['html'] = array();
    
	/* Exchange all http:// links  html */
	preg_match_all ('|<a [^>]*href="(http://[^"]*)"|Ui', $this->html, $urls);
	foreach ($urls[1] as $i => $url) {
	    $links['html'][$i] = html_entity_decode($url);
	    $this->html  = str_replace($url,"http://$this->domain/typo3conf/ext/tcdirectmail/click.php?l=$i&t=html&c=$this->cryptid", $this->html);
	}
	
	/* Exchange all http:// links plaintext */
	preg_match_all ('|http://[^ \r\n\)]*|i', $this->plain, $urls);
	foreach ($urls[0] as $i => $url) {
	    $links['plain'][$i] = html_entity_decode($url);
	    $this->plain = str_replace($url,"http://$this->domain/typo3conf/ext/tcdirectmail/click.php?l=$i&t=plain&c=$this->cryptid", $this->plain);
	}

	return $links;	
    }

    /**
     * Replace all links in the mail to make test spy links.
     * This will create links similar to the real spy links, but does not require any database activity in order to work, 
     * and does not reveal any information of the receiver.
     *
     * @return	void
     */
    function testClickLinks () {
	/* Exchange all http:// links  html */
	preg_match_all ('|<a [^>]*href="(http://[^"]*)"|Ui', $this->html, $urls);
	foreach ($urls[1] as $i => $url) {
	    $this->html  = str_replace($url,
			    "http://$this->domain/typo3conf/ext/tcdirectmail/tclick.php?l=".base64_encode(html_entity_decode($url)), 
			    $this->html);
	}
	
	/* Exchange all http:// links plaintext */
	preg_match_all ('|http://[^ \n\r\)]*|i', $this->plain, $urls);
	foreach ($urls[0] as $i => $url) {	
	    $this->plain = str_replace($url,
			    "http://$this->domain/typo3conf/ext/tcdirectmail/tclick.php?l=".base64_encode($url),
			    $this->plain);
	}
    }

    /**
     * Send multiple mails out with a Bcc-header
     *
     * @param	array		Array with email addresses.
     * @return	void
     */
    function sendMassBcc ($emails) {
	$bccHeader[] = 'Bcc: '.implode (',', $emails);
	$this->raw_send(array('email' => "Undisclosed recipients <$this->senderEmail>"), $bccHeader);
    }
    
    /**
     * The regular send method. Use this to send a normal personalized mail.
     *
     * @param	array		Record with receivers information as name => value pairs.
     * @param	array		Array with extra headers to apply to mails as name => value pairs.
     * @return	void
     */
    function send ($receiverRecord, $extraHeaders = array()) {
	$this->substituteMarkers($receiverRecord);
	
	
	$this->raw_send($receiverRecord, $extraHeaders);
	$this->resetMarkers();	
    }
    
    /**
     * Raw send method. This does not replace markers, or reset the mail afterwards.
     *
     * @interal
     * @param	array		Record with receivers information as name => value pairs.
     * @param	array		Array with extra headers to apply to mails as name => value pairs.
     * @return	void
     */
    function raw_send($receiverRecord, $extraHeaders = array()) {
	global $TYPO3_CONF_VARS;
	
	$messageId = md5(microtime().$receiverRecord['email']).'@'.$this->hostname;
	$boundary  = '----'.$this->boundarys[rand(0,count($this->boundarys)-1)].'--'.substr($messageId, 0,10);
	$subboundary  = '----'.$this->boundarys[rand(0,count($this->boundarys)-1)].'--'.substr(md5($messageId), 0,10);
	$body      = array();
	$headers   = array();
    
	$headers[] = "From: $this->senderName <$this->senderEmail>";
	$headers[] = "Message-ID: <$messageId>";
	
        $charset = $this->getCharsetEncoding();
	
	foreach ($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['tcdirectmail']['extraMailHeaders'] as $header => $value) {
	    $headers[] = "$header: $value";
	}
	
	if ($this->sendid) {
	    $headers[] = "X-tcdirectmail-sendid: $this->sendid";
	}
	if ($this->cryptid) {
	    $headers[] = "X-tcdirectmail-cryptid: $this->cryptid";
	}
	
	$headers = array_merge ($headers, $extraHeaders);
	
	$headers[] = 'MIME-Version: 1.0';
	
	/* Plain text? */
	if ($receiverRecord['plain_only'] == 1) {
	    /* Attached files? */
	    if (count($this->files)) {	
		$headers[] = "Content-Type: multipart/mixed; boundary=\"$boundary\"";
	    
		/* Standard pre-mime text */
		$body[] = 'This is a multi-part message in MIME format.';
		$body[] = '';

		/* Plain-body for mail with attached files */
    		$body[] = "--$boundary";		
		$body[] = "Content-Type: text/plain; charset=\"$charset\"";
		$body[] = 'Content-Transfer-Encoding: quoted-printable';
		$body[] = '';
		$body[] = $this->qoutedPrintableEncode(t3lib_div::substUrlsInPlainText($this->plain,'all',"http://$this->domain/"));
		$body[] = '';
		$body[] = $this->getAttachedFiles($boundary);
		$body[] = "--$boundary--";	
		
	    /* No attached files? */
	    } else {
		$headers[] = "Content-Type: text/plain; charset=\"$charset\"";
		$headers[] = 'Content-Transfer-Encoding: quoted-printable';
		$body[] = $this->qoutedPrintableEncode(t3lib_div::substUrlsInPlainText($this->plain,'all',"http://$this->domain/"));
	    }
	        
	/* or mixed */
	} else {
	    $headers[] = "Content-type: multipart/related; boundary=\"$boundary\"";	
	    
	    /* Standard pre-mime text */
	    $body[] = 'This is a multi-part message in MIME format.';
	    $body[] = '';
	    
	    /* Envelope for the text and html versions */
	    $body[] = "--$boundary";
	    $body[] = "Content-Type: multipart/alternative; boundary=\"$subboundary\"";	
	    $body[] = '';
	    
	    /* For the alternative html and text versions */
    	    $body[] = "--$subboundary";
	    $body[] = "Content-Type: text/plain; charset=\"$charset\"";
	    $body[] = 'Content-Transfer-Encoding: quoted-printable';
	    $body[] = '';
	    $body[] = $this->qoutedPrintableEncode(t3lib_div::substUrlsInPlainText($this->plain,'all',"http://$this->domain/"));
	    $body[] = '';
	    $body[] = "--$subboundary";
	    $body[] = "Content-Type: text/html; charset=\"$charset\"";
	    $body[] = 'Content-Disposition: inline';
	    $body[] = 'Content-Transfer-Encoding: base64';
	    $body[] = '';
	    $body[] = chunk_split(base64_encode($this->html));
	    $body[] = "--$subboundary--";
	    $body[] = '';
	    /* End of subboundary */
		    
	    /* Get the inline files for use with pictures, stylesheets etc. */
	    foreach ($this->inlinefiles as $filename => $content) {
		$body[] = "--$boundary";
		$body[] = 'Content-Type: '.$this->inlinemimes[$filename];
		$body[] = "Content-ID: <$filename>";
		$body[] = 'Content-Transfer-Encoding: base64';
		$body[] = '';
		$body[] = chunk_split($content);
		$body[] = '';
	    }
	    
	    /* Attach files, if any */
	    if (count($this->files)) {
		$body[] = $this->getAttachedFiles($boundary);
	    }
	    
	    $body[] = "--$boundary--";	
	}

	/* Hook for actions INSTEAD of actually sending the mail */
        if (is_array($TYPO3_CONF_VARS['EXTCONF']['tcdirectmail']['mailReplacementHook'])) {
	    foreach ($TYPO3_CONF_VARS['EXTCONF']['tcdirectmail']['mailReplacementHook'] as $_classRef) {
	        $_procObj = & t3lib_div::getUserObj($_classRef);
	        $_procObj->mailReplacementHook($receiverRecord['email'], $this->title, implode("\n", $body), implode("\n", $headers));
	    }
	} else {
    	    /* Mail it */
	    if ($TYPO3_CONF_VARS['EXTCONF']['tcdirectmail']['extraSendmailParams'] != '') {
		mail ($receiverRecord['email'], $this->title, implode("\n", $body), implode("\n", $headers), $TYPO3_CONF_VARS['EXTCONF']['tcdirectmail']['extraSendmailParams']);
	    } else {
	        mail ($receiverRecord['email'], $this->title, implode("\n", $body), implode("\n", $headers));
	    }
	}
    }
    
    /**
     * Get the MIME encoded attached files of the mail.
     *
     * @param	string		MIME boundary to encode the MIME parts with.
     * @return	string		The MIME encoded files.
     */
    function getAttachedFiles($boundary) {
	$body = array();
    
	/* Attach the files */
	foreach ($this->files as $filename => $content) {
	    $body[] = "--$boundary\nContent-Type: ".$this->mimes[$filename];
	    $body[] = " name=\"$filename\"";
	    $body[] = "Content-Disposition: attachment; filename=\"$filename\"";
	    $body[] = 'Content-Transfer-Encoding: base64';
	    $body[] = '';
	    $body[] = chunk_split($content);
	}
	
	return implode("\n", $body);    
    }
}
?>