<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2006 Daniel Schledermann <daniel@typoconsult.dk>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

/**
 * This is the tx_directmail_target's super class. 
 * All directmail targets must inherit from this class though some inheritance path.
 *
 * @abstract
 */

class tx_tcdirectmail_target {
	var $fields;
	var $data;
	
	/**
	 * This is the object factory for all directmail targets
	 *
	 * @static
	 * @param	integer		Uid of a tx:directmail_target from the database.
	 * @return 	object		Of directmail_target type.
	 */
	function loadTarget ($uid) {
	    global $TYPO3_DB;
	    
	    $rs = $TYPO3_DB->sql_query("SELECT * FROM tx_tcdirectmail_targets WHERE uid = $uid");
	    $fields = $TYPO3_DB->sql_fetch_assoc($rs);
	    $object = new $fields['targettype'];
	    if (is_subclass_of($object, 'tx_tcdirectmail_target')) {
		$object->fields = $fields;
		$object->init();
		return $object;
	    } else {
		die ("Ooops..   $fields[targettype] is not a tx_tcdirectmail_target child class");
	    }
	}
	
	/**
	 * This is the method called when a directmail_target is produced in the loadTarget factory
	 *
	 * @abstract
	 * @return 	void
	 */
	function init() {
	    die ('You need to implement the init-method.');
	}

	/**
	 * Fetch one receiver record from the directmail target.
	 *
	 * @abstract
	 * @return	array		Assoc array with fields for the receiver
	 */
	function getRecord() {
	    die ('You need to implement the getRecord-method.');
	}

	/**
	 * Reset the directmail target, so the next record being fetched will be the first. Not currently in use by any known extension
	 *
	 * @abstract
	 * @return	void
	 */
	function resetTarget() {
	    die ('You need to implement the resetTarget-method.');
	}	
	
	/**
	 * Get the number of receivers in this directmail target
	 *
	 * @abstract
	 * @return	integer		Numbers of receivers.
	 */
	function getCount() {
	    die ('You need to implement the getCount-method.');	
	}
	
	/**
	 * Get error text if the fetching of the directmail target has somehow failed.
	 *
	 * @abstract
	 * @return	string		Error text or empty string.
	 */
	function getError() {
	    die ('You need to implement the getError-method.');	
	}
	
	/**
	 * Here you can implement start events for a real send-out. This is not mandatory.
	 * 
	 * @return 	void
	 */
	function startReal() {
	}

	/**
	 * Here you can implement end events for a real send-out. This is not mandatory.
	 * 
	 * @return 	void
	 */	
	function endReal() {
	}
	
	/**
	 * Here you can implement database operation done when an email address has failed. 
	 * This is typically disabling of the record. 
	 * This might be used in bound handling.
	 * This is not mandatory
	 *
	 * @param	string		Email address that has failed.
	 * @return	bool		Status of the success of the removal.
	 */
	function disableEmailAddress($email) {
	    return false;
	}
}

?>